﻿Public Class Class1
    Public SerialPort1 As New System.IO.Ports.SerialPort

    Public Stop_Flg As Boolean = False
    Public WR_Doing As Boolean = False

    Public Sub New()
        SerialPort1.PortName = "COM3"
        SerialPort1.BaudRate = 38400
        SerialPort1.Parity = IO.Ports.Parity.None
        SerialPort1.DataBits = 8
        SerialPort1.StopBits = 1
        SerialPort1.Handshake = IO.Ports.Handshake.None
        SerialPort1.NewLine = vbCrLf
    End Sub


    'Connect
    Public Function Connect() As Boolean
        Dim buf As String
        Dim data() As String

        Try
            If SerialPort1.IsOpen = False Then
                SerialPort1.Open()                              'Port Open
                SerialPort1.WriteLine("*IDN?")                  'Send *IDN?
                Threading.Thread.Sleep(50)
                buf = SerialPort1.ReadLine()                    'Recive
                data = buf.Split(",")
                If data(1) <> "SHRC-203" Then Throw New Exception   'Check the recive data

                If GetAxisCnt() = False Then Throw New Exception    'Get AxisCount

                Wait_Ready()  'Show current position
            End If
        Catch ex As Exception
            SerialPort1.Close()
            Return False
        End Try
        Return True
    End Function

    'Disconnect
    Public Function Disconnect() As Boolean
        Try
            SerialPort1.Close() 'Port Close
        Catch ex As Exception
            Return False
        End Try
        Return True
    End Function

    'Get axis count
    Public Function GetAxisCnt() As Boolean
        Dim buf As String

        Try
            If SerialPort1.IsOpen = True Then
                SerialPort1.WriteLine("?:AXIS")              'Send ?:AXIS
                Threading.Thread.Sleep(50)
                buf = SerialPort1.ReadLine()                 'Recive
                AxisCount = CInt(Mid(buf, 1, 1))
            Else
                Throw New Exception
            End If
        Catch ex As Exception
            AxisCount = -1
            Return False
        End Try
        Return True
    End Function

    'Get Speed
    Public Function GetSpeed(ByRef Speed As String) As Boolean
        Dim buf, buf2 As String

        Try
            If SerialPort1.IsOpen = True Then
                buf = "?:D"
                SerialPort1.WriteLine(buf)  'Send ? command
                Threading.Thread.Sleep(50)
                buf2 = SerialPort1.ReadLine 'Recive
                Speed = buf2
            Else
                Throw New Exception
            End If
        Catch ex As Exception
            Return False
        End Try
        Return True
    End Function

    'Wait to Ready
    Public Function Wait_Ready() As Boolean
        Dim i As Integer
        Dim st As String
        Dim buf As String
        Dim buf2() As String = Nothing

        Try
            If SerialPort1.IsOpen = True Then
                WR_Doing = True
                Do
                    System.Windows.Forms.Application.DoEvents()
                    If Stop_Flg = True Then
                        SerialPort1.DiscardInBuffer()   'Clear buffer
                        SerialPort1.WriteLine("L:W")    'Send L command
                        Threading.Thread.Sleep(50)
                        buf = SerialPort1.ReadLine      'Recive
                        If InStr(1, UCase(buf), "OK") = 0 Then Throw New Exception
                        Exit Do
                    End If
                    SerialPort1.DiscardInBuffer()   'Clear buffer
                    SerialPort1.WriteLine("Q:")     'Send Q command
                    Threading.Thread.Sleep(50)
                    buf = SerialPort1.ReadLine      'Recive
                    buf2 = buf.Split(",")
                    For i = 0 To 2
                        pLabel_Position(i).Text = buf2(i)
                    Next
                    System.Windows.Forms.Application.DoEvents()
                    st = Mid(buf2(5), 1, 1).ToUpper
                    If st = "R" Then                'Check Ready
                        Return True
                    End If
                Loop While True
            Else
                Throw New Exception
            End If
        Catch ex As Exception
            Return False
        Finally
            Stop_Flg = False
            WR_Doing = False
        End Try
        Return True
    End Function

    'Set Speed
    Public Function SetSpeed(Speed() As String) As Boolean
        Dim i As Integer
        Dim buf, buf2 As String

        Try
            If SerialPort1.IsOpen = True Then
                buf = "D:" + GetAxisEnable()
                For i = 0 To 2
                    If pCheckBoxEnable(i).Checked = True Then
                        buf = buf + "S" + Speed(0) + "F" + Speed(1) + "R" + Speed(2)
                    End If
                Next
                SerialPort1.WriteLine(buf)  'Send D command
                Threading.Thread.Sleep(50)
                buf2 = SerialPort1.ReadLine 'Recive
                If InStr(1, UCase(buf2), "OK") = 0 Then Throw New Exception
            Else
                Throw New Exception
            End If
        Catch ex As Exception
            Return False
        End Try
        Return True
    End Function

    'Retrun Origin
    Public Function ReturnOrigin() As Boolean
        Dim buf, buf2 As String
        Try
            If SerialPort1.IsOpen = True Then
                buf = "H:" + GetAxisEnable()
                SerialPort1.WriteLine(buf)  'Send H command
                Threading.Thread.Sleep(50)
                buf2 = SerialPort1.ReadLine 'Recive
                If InStr(1, UCase(buf2), "OK") = 0 Then Throw New Exception
                Threading.Thread.Sleep(50)
                Wait_Ready()  'Show current position
            Else
                Throw New Exception
            End If
        Catch ex As Exception
            Return False
        End Try
        Return True
    End Function

    'Stop
    Public Function StopStage() As Boolean
        Dim buf, buf2 As String

        If WR_Doing = True Then 'Already doing
            Stop_Flg = True
            Return True
        End If
        Try
            If SerialPort1.IsOpen = True Then
                buf = "L:" + GetAxisEnable()
                SerialPort1.WriteLine(buf)  'Send L command
                Threading.Thread.Sleep(50)
                buf2 = SerialPort1.ReadLine 'Recive
                If InStr(1, UCase(buf2), "OK") = 0 Then Throw New Exception
                Threading.Thread.Sleep(50)
                Wait_Ready()  'Show current position
            Else
                Throw New Exception
            End If
        Catch ex As Exception
            Return False
        End Try
        Return True
    End Function

    'Stop Emergency
    Public Function StopStageEmergency() As Boolean
        Dim buf, buf2 As String

        Try
            If SerialPort1.IsOpen = True Then
                buf = "L:E"
                SerialPort1.WriteLine(buf)  'Send L command
                Threading.Thread.Sleep(50)
                buf2 = SerialPort1.ReadLine 'Recive
                If InStr(1, UCase(buf2), "OK") = 0 Then Throw New Exception
                Threading.Thread.Sleep(50)
                Wait_Ready()  'Show current position
            Else
                Throw New Exception
            End If
        Catch ex As Exception
            Return False
        End Try
        Return True
    End Function

    'Reset
    Public Function ResetPosition() As Boolean
        Dim buf, buf2 As String

        Try
            If SerialPort1.IsOpen = True Then
                buf = "R:" + GetAxisEnable()
                SerialPort1.WriteLine(buf)  'Send R command
                Threading.Thread.Sleep(50)
                buf2 = SerialPort1.ReadLine 'Recive
                If InStr(1, UCase(buf2), "OK") = 0 Then Throw New Exception
                Threading.Thread.Sleep(50)
                Wait_Ready()  'Show current position
            Else
                Throw New Exception
            End If
        Catch ex As Exception
            Return False
        End Try
        Return True
    End Function

    'JOG
    Public Function JOG(Dir As Boolean) As Boolean
        Dim i As Integer
        Dim buf, buf2 As String
        Dim dirStr As String

        Try
            If SerialPort1.IsOpen = True Then
                buf = "J:" + GetAxisEnable()
                If Dir = True Then
                    dirStr = "+"
                Else
                    dirStr = "-"
                End If
                buf2 = ""
                For i = 0 To 2
                    If pCheckBoxEnable(i).Checked = True Then
                        buf2 = buf2 + dirStr
                    End If
                Next
                buf = buf + buf2
                SerialPort1.WriteLine(buf)  'Send J command
                Threading.Thread.Sleep(50)
                buf2 = ""
                buf2 = SerialPort1.ReadLine 'Recive
                If InStr(1, UCase(buf2), "OK") = 0 Then Throw New Exception
                Threading.Thread.Sleep(50)

                buf = "G:"
                SerialPort1.WriteLine(buf)  'Send G command
                Threading.Thread.Sleep(50)
                buf2 = ""
                buf2 = SerialPort1.ReadLine 'Recive
                If InStr(1, UCase(buf2), "OK") = 0 Then Throw New Exception
                Threading.Thread.Sleep(50)

                Wait_Ready()  'Show current position
            Else
                Throw New Exception
            End If
        Catch ex As Exception
            Return False
        End Try
        Return True
    End Function

    'Move
    Public Function Move(Value() As Integer) As Boolean
        Dim i As Integer
        Dim buf, buf2 As String
        Dim uuu(2) As String

        Try
            If SerialPort1.IsOpen = True Then
                buf = "M:" + GetAxisEnable()
                buf2 = ""
                For i = 0 To 2
                    If Value(i) < 0 Then
                        uuu(i) = "-"
                    Else
                        uuu(i) = "+"
                    End If
                    Value(i) = Math.Abs(Value(i))
                    If pCheckBoxEnable(i).Checked = True Then
                        buf2 = buf2 + uuu(i) + "P" + Value(i).ToString
                    End If
                Next
                buf = buf + buf2
                SerialPort1.WriteLine(buf)  'Send M command
                Threading.Thread.Sleep(50)
                buf2 = ""
                buf2 = SerialPort1.ReadLine 'Recive
                If InStr(1, UCase(buf2), "OK") = 0 Then Throw New Exception
                Threading.Thread.Sleep(50)

                buf = "G:"
                SerialPort1.WriteLine(buf)  'Send G command
                Threading.Thread.Sleep(50)
                buf2 = ""
                buf2 = SerialPort1.ReadLine 'Recive
                If InStr(1, UCase(buf2), "OK") = 0 Then Throw New Exception
                Threading.Thread.Sleep(50)

                Wait_Ready()  'Show current position
            Else
                Throw New Exception
            End If
        Catch ex As Exception
            Return False
        End Try
        Return True
    End Function

    'Move Absolute
    Public Function MoveAbs(Value() As Integer) As Boolean
        Dim i As Integer
        Dim buf, buf2 As String
        Dim uuu(2) As String

        Try
            If SerialPort1.IsOpen = True Then
                buf = "A:" + GetAxisEnable()
                buf2 = ""
                For i = 0 To 2
                    If Value(i) < 0 Then
                        uuu(i) = "-"
                    Else
                        uuu(i) = "+"
                    End If
                    Value(i) = Math.Abs(Value(i))
                    If pCheckBoxEnable(i).Checked = True Then
                        buf2 = buf2 + uuu(i) + "P" + Value(i).ToString
                    End If
                Next
                buf = buf + buf2
                SerialPort1.WriteLine(buf)  'Send A command
                Threading.Thread.Sleep(50)
                buf2 = ""
                buf2 = SerialPort1.ReadLine 'Recive
                If InStr(1, UCase(buf2), "OK") = 0 Then Throw New Exception
                Threading.Thread.Sleep(50)

                buf = "G:"
                SerialPort1.WriteLine(buf)  'Send G command
                Threading.Thread.Sleep(50)
                buf2 = ""
                buf2 = SerialPort1.ReadLine 'Recive
                If InStr(1, UCase(buf2), "OK") = 0 Then Throw New Exception
                Threading.Thread.Sleep(50)

                Wait_Ready()  'Show current position
            Else
                Throw New Exception
            End If
        Catch ex As Exception
            Return False
        End Try
        Return True
    End Function


    'Get B/R
    Public Function GetBR() As String
        Dim st As String
        Dim buf As String
        Dim buf2() As String = Nothing

        Try
            If SerialPort1.IsOpen = True Then
                SerialPort1.WriteLine("Q:")     'Send Q command
                Threading.Thread.Sleep(50)
                buf = SerialPort1.ReadLine      'Recive
                buf2 = buf.Split(",")
                st = Mid(buf2(5), 1, 1).ToUpper 'B/R
            Else
                Throw New Exception
            End If
        Catch ex As Exception
            Return ""
        End Try
        Return st
    End Function


    'Get Position (Axis = 1 - 3)
    Public Function GetPosition(Axis As Integer) As String
        Dim i As Integer
        Dim st As String = ""
        Dim buf As String
        Dim buf2() As String = Nothing

        Try
            If SerialPort1.IsOpen = True Then
                SerialPort1.WriteLine("Q:")     'Send Q command
                Threading.Thread.Sleep(50)
                buf = SerialPort1.ReadLine      'Recive
                buf2 = buf.Split(",")
                st = buf2(Axis - 1)             'Position
            End If
        Catch ex As Exception
            Return ""
        End Try
        Return st
    End Function



End Class
